(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/http'), require('rxjs/Observable'), require('rxjs/add/operator/catch'), require('rxjs/add/operator/map'), require('@angular/core')) :
	typeof define === 'function' && define.amd ? define(['exports', '@angular/http', 'rxjs/Observable', 'rxjs/add/operator/catch', 'rxjs/add/operator/map', '@angular/core'], factory) :
	(factory((global.ng = global.ng || {}, global.ng['angular-api-service'] = global.ng['angular-api-service'] || {}),global._angular_http,global.Rx,null,global.Rx.Observable.prototype,global.ng.core));
}(this, (function (exports,_angular_http,rxjs_Observable,rxjs_add_operator_catch,rxjs_add_operator_map,_angular_core) { 'use strict';

var Filter = (function () {
    function Filter() {
    }
    return Filter;
}());

var AliasMapFactory = (function () {
    function AliasMapFactory(aliases) {
        this.aliases = aliases;
    }
    AliasMapFactory.prototype.create = function () {
        var map = new Map();
        var keys = Object.getOwnPropertyNames(this.aliases);
        for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
            var key = keys_1[_i];
            map.set(key, this.aliases[key]);
        }
        return map;
    };
    return AliasMapFactory;
}());

var Transformer = (function () {
    /**
     *
     * @param transformersMap
     * @param withRelations
     */
    function Transformer(transformersMap, withRelations) {
        if (withRelations === void 0) { withRelations = true; }
        this.transformersMap = transformersMap;
        this.withRelations = withRelations;
        // If aliases present then convert them to aliases map.
        var aliases = this.aliases();
        if (aliases)
            this.aliasesMap = Transformer.createAliasesMap(aliases);
    }
    /**
     * Create aliases map.
     *
     * @param aliases
     * @returns {AliasMap}
     */
    Transformer.createAliasesMap = function (aliases) {
        return new AliasMapFactory(aliases).create();
    };
    /**
     * Transform response data to model entity.
     *
     * @param response
     * @returns {Model}
     */
    Transformer.prototype.call = function (response) {
        this.response = response;
        // List
        if (response.data && response.data instanceof Array)
            return this.response.data.map(this.parse, this);
        else if (response.data)
            return this.parse(response.data);
        else if (response.attributes)
            return this.parse(response);
    };
    /**
     * Parse main model data.
     *
     * @param data
     * @returns {Model}
     */
    Transformer.prototype.parse = function (data) {
        var model = this.transform(data);
        if (this.withRelations && 'relationships' in data && this.transformersMap)
            this.parseRelations(data.relationships, model);
        return model;
    };
    /**
     * Parse relations list.
     *
     * @param relations
     * @param model
     */
    Transformer.prototype.parseRelations = function (relations, model) {
        var keys = Object.getOwnPropertyNames(relations);
        for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
            var key = keys_1[_i];
            var relation = relations[key].data;
            if (relation instanceof Array) {
                for (var _a = 0, relation_1 = relation; _a < relation_1.length; _a++) {
                    var _relation = relation_1[_a];
                    this.parseRelation(_relation, model, true);
                }
            }
            else
                this.parseRelation(relation, model);
        }
    };
    /**
     * Parse the relation.
     *
     * @param data
     * @param model
     * @param isArray
     */
    Transformer.prototype.parseRelation = function (data, model, isArray) {
        if (isArray === void 0) { isArray = false; }
        var alias = this.findAlias(data.type);
        var type = data.type;
        var include = this.findInclude(type, data.id);
        this.parseInclude(include, model, alias, isArray);
    };
    /**
     * Find an alias for model type.
     *
     * @param type
     * @returns {undefined|string}
     */
    Transformer.prototype.findAlias = function (type) {
        if (this.aliasesMap)
            return this.aliasesMap.get(type);
    };
    /**
     * Find model include data.
     *
     * @param type
     * @param id
     * @returns {any}
     */
    Transformer.prototype.findInclude = function (type, id) {
        for (var _i = 0, _a = this.response.included; _i < _a.length; _i++) {
            var include = _a[_i];
            if (include.type == type && include.id == id)
                return include;
        }
    };
    /**
     * Parse model include data.
     *
     * @param include
     * @param model
     * @param alias
     * @param isArray
     */
    Transformer.prototype.parseInclude = function (include, model, alias, isArray) {
        if (isArray === void 0) { isArray = false; }
        if (!include.type && !alias)
            throw new Error('Include item type or alias must be set. Please check your backend API response.');
        var type = include.type;
        var transformer = this.transformersMap.get(alias || type);
        var includeModel = transformer ? transformer.transform(include) : include;
        // Parse sub include relations if exists
        if (include.relationships)
            this.parseRelations(include.relationships, includeModel);
        if (isArray) {
            if (!model[type])
                model[type] = [];
            model[type].push(includeModel);
        }
        else
            model[type] = includeModel;
    };
    return Transformer;
}());

var Search = (function () {
    function Search() {
        this.operator = '=';
    }
    Search.create = function (field, value, operator) {
        var instance = new Search();
        instance.field = field;
        instance.value = value;
        if (operator)
            instance.operator = operator;
        return instance;
    };
    return Search;
}());

var Factory = (function () {
    function Factory() {
    }
    Factory.TYPE_QUERY = 'query';
    Factory.TYPE_FIELDS = 'fields';
    return Factory;
}());

var Order = (function () {
    function Order() {
    }
    Order.create = function (field, sort, customKey, table) {
        var order = new Order();
        order.field = field;
        order.sort = sort;
        order.customKey = customKey;
        order.table = table;
        return order;
    };
    return Order;
}());

var Relation = (function () {
    function Relation() {
    }
    Relation.create = function (name) {
        var relation = new Relation();
        relation.relation = name;
        return relation;
    };
    return Relation;
}());

var Collection = (function () {
    function Collection(items) {
        if (items === void 0) { items = []; }
        this.items = items;
    }
    Collection.prototype.add = function (item) {
        if (item instanceof Array)
            return (_a = this.items).push.apply(_a, item);
        else
            return this.items.push(item);
        var _a;
    };
    Collection.prototype.remove = function (item) {
        if (item instanceof Number)
            return this.items.splice(item, 1);
        else {
            var index = this.items.indexOf(item);
            if (index !== -1)
                return this.items.splice(index, 1);
            else
                return this.items;
        }
    };
    Collection.prototype.get = function (index) {
        return this.items[index];
    };
    Collection.prototype.all = function () {
        return this.items;
    };
    Collection.prototype.first = function () {
        return this.items[0];
    };
    Collection.prototype.last = function () {
        return this.items[this.items.length - 1];
    };
    Collection.prototype.count = function () {
        return this.items.length;
    };
    Collection.prototype.has = function (item) {
        if (item instanceof Number)
            return item in this.items;
        else
            return this.items.indexOf(item) >= 0;
    };
    return Collection;
}());

var __extends = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var ApiCollection = (function (_super) {
    __extends(ApiCollection, _super);
    function ApiCollection() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return ApiCollection;
}(Collection));

var ApiResponse = (function () {
    function ApiResponse(service, response) {
        this.service = service;
        this.response = response;
    }
    ApiResponse.prototype.transform = function (data) {
        var transformer = this.service.transformer();
        return transformer.call(data);
    };
    ApiResponse.prototype.fetch = function () {
        return this.transform(this.getResponse());
    };
    ApiResponse.prototype.getPagination = function () {
        return this.getMeta().pagination;
    };
    ApiResponse.prototype.getLinks = function () {
        return this.getResponse().links;
    };
    ApiResponse.prototype.getIncluded = function () {
        return this.getResponse().included;
    };
    ApiResponse.prototype.getMeta = function () {
        return this.getResponse().meta;
    };
    ApiResponse.prototype.getData = function () {
        return this.getResponse().data;
    };
    ApiResponse.prototype.getResponse = function () {
        return this.response;
    };
    return ApiResponse;
}());

var __extends$2 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var SearchCollection = (function (_super) {
    __extends$2(SearchCollection, _super);
    function SearchCollection() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(SearchCollection, "separator", {
        get: function () {
            return ';';
        },
        enumerable: true,
        configurable: true
    });
    SearchCollection.prototype.toUrl = function () {
        if (this.count() === 1)
            return this.first().value;
        else
            return this.all()
                .map(function (item) {
                return item.field + ":" + item.value;
            })
                .join(SearchCollection.separator);
    };
    return SearchCollection;
}(ApiCollection));

var __extends$3 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var SearchFieldCollection = (function (_super) {
    __extends$3(SearchFieldCollection, _super);
    function SearchFieldCollection() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SearchFieldCollection.prototype.toUrl = function () {
        return this.all()
            .map(function (item) {
            return item.operator !== SearchFieldCollection.defaultOperator ? item.field + ":" + item.operator : null;
        })
            .filter(function (item) {
            return item;
        })
            .join(SearchFieldCollection.separator);
    };
    SearchFieldCollection.separator = ';';
    SearchFieldCollection.defaultOperator = '=';
    return SearchFieldCollection;
}(ApiCollection));

var __extends$1 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var SearchFactory = (function (_super) {
    __extends$1(SearchFactory, _super);
    function SearchFactory(search) {
        var _this = _super.call(this) || this;
        if (!(search instanceof Search || search instanceof Array))
            throw new TypeError('Search argument must be a type "Search" or "Search[]".');
        if (!(search instanceof Array))
            _this.search = [search];
        else
            _this.search = search;
        return _this;
    }
    SearchFactory.prototype.create = function (type) {
        switch (type) {
            case SearchFactory.TYPE_QUERY:
                return new SearchCollection(this.search);
            case SearchFactory.TYPE_FIELDS:
                return new SearchFieldCollection(this.search);
            default:
                throw new Error('Search factory support only: "query" or "fields" type.');
        }
    };
    return SearchFactory;
}(Factory));

var __extends$5 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var OrderCollection = (function (_super) {
    __extends$5(OrderCollection, _super);
    function OrderCollection() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    OrderCollection.prototype.toUrl = function () {
        return this.all()
            .map(function (item) {
            if (item.table && item.customKey && item.field)
                return item.table + ":" + item.customKey + "|" + item.field;
            else if (item.table && item.field)
                return item.table + "|" + item.field;
            else
                return item.field;
        })
            .join(OrderCollection.separator);
    };
    OrderCollection.separator = ';';
    return OrderCollection;
}(ApiCollection));

var __extends$6 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var SortCollection = (function (_super) {
    __extends$6(SortCollection, _super);
    function SortCollection() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SortCollection.prototype.toUrl = function () {
        return this.all()
            .map(function (item) {
            return item.sort != SortCollection.defaultSort ? item.field + ":" + (item.sort == 1 /* DESC */ ? 'desc' : 'asc') : null;
        })
            .join(SortCollection.separator);
    };
    SortCollection.separator = ';';
    SortCollection.defaultSort = 0 /* ASC */;
    return SortCollection;
}(ApiCollection));

var __extends$4 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var OrderFactory = (function (_super) {
    __extends$4(OrderFactory, _super);
    function OrderFactory(order) {
        var _this = _super.call(this) || this;
        if (!(order instanceof Order || order instanceof Array))
            throw new TypeError('Order argument must be a type "Order" or "Order[]".');
        if (!(order instanceof Array))
            _this.order = [order];
        else
            _this.order = order;
        return _this;
    }
    OrderFactory.prototype.create = function (type) {
        switch (type) {
            case OrderFactory.TYPE_QUERY:
                return new OrderCollection(this.order);
            case OrderFactory.TYPE_FIELDS:
                return new SortCollection(this.order);
            default:
                throw new Error('Order factory support only: "query" or "fields" type.');
        }
    };
    return OrderFactory;
}(Factory));

var __extends$8 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var FilterCollection = (function (_super) {
    __extends$8(FilterCollection, _super);
    function FilterCollection() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    FilterCollection.prototype.toUrl = function () {
        return this.all()
            .map(function (item) {
            return item.value;
        })
            .join(FilterCollection.separator);
    };
    FilterCollection.separator = ';';
    return FilterCollection;
}(ApiCollection));

var __extends$7 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var FilterFactory = (function (_super) {
    __extends$7(FilterFactory, _super);
    function FilterFactory(filters) {
        var _this = _super.call(this) || this;
        if (!(filters instanceof Search || filters instanceof Array))
            throw new TypeError('Search argument must be a type "Search" or "Search[]".');
        if (!(filters instanceof Array))
            filters = [filters];
        _this.filters = filters;
        return _this;
    }
    FilterFactory.prototype.create = function (type) {
        switch (type) {
            case FilterFactory.TYPE_FIELDS:
                return new FilterCollection(this.filters);
            default:
                throw new Error('Filter factory support only: "fields" type.');
        }
    };
    return FilterFactory;
}(Factory));

var __extends$9 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var RelationCollection = (function (_super) {
    __extends$9(RelationCollection, _super);
    function RelationCollection() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(RelationCollection, "separator", {
        get: function () {
            return ',';
        },
        enumerable: true,
        configurable: true
    });
    RelationCollection.prototype.toUrl = function () {
        return this.all()
            .map(function (item) {
            return item.relation;
        })
            .join(RelationCollection.separator);
    };
    return RelationCollection;
}(ApiCollection));

var ApiService = (function () {
    function ApiService(http) {
        this.http = http;
    }
    ApiService.prototype.extractData = function (res) {
        return new ApiResponse(this, res.json() || {});
    };
    ApiService.prototype.handleError = function (error) {
        var errMsg;
        if (error instanceof _angular_http.Response) {
            var body = error.json() || '';
            var err = body.error || JSON.stringify(body);
            errMsg = error.status + " - " + (error.statusText || '') + " " + err;
        }
        else {
            errMsg = error.message ? error.message : error.toString();
        }
        console.error(errMsg);
        return rxjs_Observable.Observable.throw(errMsg);
        // return 'custom error';
    };
    /*****************************************************************************************************************
     *      Http client methods aliases.
     */
    ApiService.prototype.get = function (url, search, orders, filters, relations, options) {
        if (!options)
            options = new _angular_http.RequestOptions();
        var params = {};
        if (search) {
            var factory = new SearchFactory(search);
            params[ApiService.urlSearchQueryParam] = factory.create(SearchFactory.TYPE_QUERY).toUrl();
            params[ApiService.urlSearchFieldsParam] = factory.create(SearchFactory.TYPE_FIELDS).toUrl();
        }
        if (orders) {
            var factory = new OrderFactory(orders);
            params[ApiService.urlOrderParam] = factory.create(OrderFactory.TYPE_QUERY).toUrl();
            params[ApiService.urlSortParam] = factory.create(OrderFactory.TYPE_FIELDS).toUrl();
        }
        if (filters)
            params[ApiService.urlFilterParam] = new FilterFactory(filters).create(FilterFactory.TYPE_FIELDS).toUrl();
        if (relations)
            params[ApiService.urlIncludeParam] = new RelationCollection(relations).toUrl();
        options.params = Object.assign({}, params, options.params);
        // hack for using service context instance
        var self = this;
        return this.http.get(url, options)
            .map(function (res) { return self.extractData(res); })
            .catch(function (error) { return self.handleError(error); });
    };
    ApiService.prototype.post = function (url, body, options) {
        // hack for using service context instance
        var self = this;
        return this.http.post(url, body, options)
            .map(function (res) { return self.extractData(res); })
            .catch(function (error) { return self.handleError(error); });
    };
    ApiService.prototype.put = function (url, body, options) {
        var self = this;
        return this.http.put(url, body, options)
            .map(this.extractData)
            .catch(function (error) { return self.handleError(error); });
    };
    ApiService.prototype.patch = function (url, body, options) {
        var self = this;
        return this.http.patch(url, body, options)
            .map(this.extractData)
            .catch(function (error) { return self.handleError(error); });
    };
    ApiService.prototype.delete = function (url, options) {
        var self = this;
        return this.http.delete(url, options)
            .map(this.extractData)
            .catch(function (error) { return self.handleError(error); });
    };
    ApiService.prototype.head = function (url, options) {
        return this.http.head(url, options);
    };
    ApiService.prototype.options = function (url, options) {
        return this.http.options(url, options);
    };
    ApiService.urlSearchQueryParam = 'search';
    ApiService.urlSearchFieldsParam = 'searchFields';
    ApiService.urlOrderParam = 'orderBy';
    ApiService.urlSortParam = 'sortedBy';
    ApiService.urlFilterParam = 'filter';
    ApiService.urlIncludeParam = 'include';
    return ApiService;
}());

var __decorate$1 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var TRANSFORMERS_DATA_MAP = {};
var TransformerMap = (function () {
    function TransformerMap(injector) {
        this.injector = injector;
    }
    TransformerMap_1 = TransformerMap;
    /**
     * Put the transformer entry to map.
     *
     * @param args
     */
    TransformerMap.prototype.put = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var key = args[0];
        var entryToken = args[1];
        var transformerClass = args[2];
        var entry;
        if (entryToken instanceof _angular_core.InjectionToken)
            entry = TransformerMap_1.createEntry(entryToken, transformerClass);
        else
            entry = entryToken;
        TransformerMap_1.map[key] = entry;
    };
    /**
     * Create transformer map entry.
     *
     * @param token Injection token.
     * @param transformerClass Transformer class.
     * @returns {ITransformerDataMapEntry}
     */
    TransformerMap.createEntry = function (token, transformerClass) {
        return {
            token: token,
            transformerClass: transformerClass
        };
    };
    /**
     * Inject and get the transformer instance by key.
     *
     * @param key
     * @returns {ITransformer<IEntity>}
     */
    TransformerMap.prototype.get = function (key) {
        try {
            return this.injector.get(TransformerMap_1.map[key].token);
        }
        catch (e) {
            throw new Error("Can't inject transformer by key \"" + key + "\". Maybe transformer not registered in the map.");
        }
    };
    TransformerMap.map = TRANSFORMERS_DATA_MAP;
    TransformerMap = TransformerMap_1 = __decorate$1([
        _angular_core.Injectable()
    ], TransformerMap);
    return TransformerMap;
    var TransformerMap_1;
}());
var TRANSFORMERS_MAP = new _angular_core.InjectionToken('app.transformers-map');

var __decorate = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var TransformerModule = (function () {
    function TransformerModule() {
    }
    TransformerModule_1 = TransformerModule;
    TransformerModule.extend = function (map, withProviders) {
        if (withProviders === void 0) { withProviders = true; }
        Object.assign(TRANSFORMERS_DATA_MAP, map);
        if (withProviders)
            this.extendProviders(map);
        return {
            ngModule: TransformerModule_1,
            providers: this.providers
        };
    };
    TransformerModule.extendProviders = function (map) {
        var keys = Object.getOwnPropertyNames(map);
        var _loop_1 = function (key) {
            var provider = {
                provide: map[key].token,
                useFactory: function (transformersMap) {
                    return new map[key].transformerClass(transformersMap);
                },
                deps: [TRANSFORMERS_MAP]
            };
            this_1.providers.push(provider);
        };
        var this_1 = this;
        for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
            var key = keys_1[_i];
            _loop_1(key);
        }
    };
    TransformerModule.providers = [{
            provide: TRANSFORMERS_MAP,
            useFactory: function (injector) {
                return new TransformerMap(injector);
            },
            deps: [_angular_core.Injector]
        }];
    TransformerModule = TransformerModule_1 = __decorate([
        _angular_core.NgModule({
            providers: TransformerModule_1.providers
        })
    ], TransformerModule);
    return TransformerModule;
    var TransformerModule_1;
}());

exports.Filter = Filter;
exports.Transformer = Transformer;
exports.Search = Search;
exports.Factory = Factory;
exports.Order = Order;
exports.Relation = Relation;
exports.ApiCollection = ApiCollection;
exports.ApiService = ApiService;
exports.ApiResponse = ApiResponse;
exports.Collection = Collection;
exports.FilterCollection = FilterCollection;
exports.FilterFactory = FilterFactory;
exports.OrderCollection = OrderCollection;
exports.OrderFactory = OrderFactory;
exports.RelationCollection = RelationCollection;
exports.SearchCollection = SearchCollection;
exports.SearchFactory = SearchFactory;
exports.SearchFieldCollection = SearchFieldCollection;
exports.SortCollection = SortCollection;
exports.TransformerModule = TransformerModule;
exports.TRANSFORMERS_DATA_MAP = TRANSFORMERS_DATA_MAP;
exports.TransformerMap = TransformerMap;
exports.TRANSFORMERS_MAP = TRANSFORMERS_MAP;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=angular-api-service.umd.js.map
