import { IApiService } from "./iapi-service";
import { Http, RequestOptionsArgs, Response } from "@angular/http";
import { ApiResponse } from "./api-response";
import { Observable } from 'rxjs/Observable';
import 'rxjs/add/operator/catch';
import 'rxjs/add/operator/map';
import { ITransformer } from "./itransformer";
import { Search } from "./search";
import { Order } from "./order";
import { Filter } from "./filter";
import { Relation } from "./relation";
import { ErrorObservable } from "rxjs/observable/ErrorObservable";
import { IEntity } from "./ientity";
export declare abstract class ApiService<Model extends IEntity> implements IApiService<Model> {
    private http;
    static readonly urlSearchQueryParam: string;
    static readonly urlSearchFieldsParam: string;
    static readonly urlOrderParam: string;
    static readonly urlSortParam: string;
    static readonly urlFilterParam: string;
    static readonly urlIncludeParam: string;
    constructor(http: Http);
    protected extractData(res: Response): any;
    protected handleError(error: Response | any): ErrorObservable;
    abstract transformer(): ITransformer<Model>;
    /*****************************************************************************************************************
     *      Http client methods aliases.
     */
    get(url: string, search?: Search[], orders?: Order[], filters?: Filter[], relations?: Relation[], options?: RequestOptionsArgs): Observable<ApiResponse<Model>>;
    post(url: string, body: Model | any, options?: RequestOptionsArgs): Observable<ApiResponse<Model>>;
    put(url: string, body: Model | any, options?: RequestOptionsArgs): Observable<ApiResponse<Model>>;
    patch(url: string, body: Model | any, options?: RequestOptionsArgs): Observable<ApiResponse<Model>>;
    delete(url: string, options?: RequestOptionsArgs): Observable<ApiResponse<Model>>;
    head(url: string, options?: RequestOptionsArgs): Observable<Response>;
    options(url: string, options?: RequestOptionsArgs): Observable<Response>;
}
