import { RequestOptions, Response } from "@angular/http";
import { ApiResponse } from "./api-response";
import { Observable } from 'rxjs/Observable';
import 'rxjs/add/operator/catch';
import 'rxjs/add/operator/map';
import { SearchFactory } from "./search.factory";
import { OrderFactory } from "./order.factory";
import { FilterFactory } from "./filter.factory";
import { RelationCollection } from "./relation.collection";
var ApiService = (function () {
    function ApiService(http) {
        this.http = http;
    }
    ApiService.prototype.extractData = function (res) {
        return new ApiResponse(this, res.json() || {});
    };
    ApiService.prototype.handleError = function (error) {
        var errMsg;
        if (error instanceof Response) {
            var body = error.json() || '';
            var err = body.error || JSON.stringify(body);
            errMsg = error.status + " - " + (error.statusText || '') + " " + err;
        }
        else {
            errMsg = error.message ? error.message : error.toString();
        }
        console.error(errMsg);
        return Observable.throw(errMsg);
        // return 'custom error';
    };
    /*****************************************************************************************************************
     *      Http client methods aliases.
     */
    ApiService.prototype.get = function (url, search, orders, filters, relations, options) {
        if (!options)
            options = new RequestOptions();
        var params = {};
        if (search) {
            var factory = new SearchFactory(search);
            params[ApiService.urlSearchQueryParam] = factory.create(SearchFactory.TYPE_QUERY).toUrl();
            params[ApiService.urlSearchFieldsParam] = factory.create(SearchFactory.TYPE_FIELDS).toUrl();
        }
        if (orders) {
            var factory = new OrderFactory(orders);
            params[ApiService.urlOrderParam] = factory.create(OrderFactory.TYPE_QUERY).toUrl();
            params[ApiService.urlSortParam] = factory.create(OrderFactory.TYPE_FIELDS).toUrl();
        }
        if (filters)
            params[ApiService.urlFilterParam] = new FilterFactory(filters).create(FilterFactory.TYPE_FIELDS).toUrl();
        if (relations)
            params[ApiService.urlIncludeParam] = new RelationCollection(relations).toUrl();
        options.params = Object.assign({}, params, options.params);
        // hack for using service context instance
        var self = this;
        return this.http.get(url, options)
            .map(function (res) { return self.extractData(res); })
            .catch(function (error) { return self.handleError(error); });
    };
    ApiService.prototype.post = function (url, body, options) {
        // hack for using service context instance
        var self = this;
        return this.http.post(url, body, options)
            .map(function (res) { return self.extractData(res); })
            .catch(function (error) { return self.handleError(error); });
    };
    ApiService.prototype.put = function (url, body, options) {
        var self = this;
        return this.http.put(url, body, options)
            .map(this.extractData)
            .catch(function (error) { return self.handleError(error); });
    };
    ApiService.prototype.patch = function (url, body, options) {
        var self = this;
        return this.http.patch(url, body, options)
            .map(this.extractData)
            .catch(function (error) { return self.handleError(error); });
    };
    ApiService.prototype.delete = function (url, options) {
        var self = this;
        return this.http.delete(url, options)
            .map(this.extractData)
            .catch(function (error) { return self.handleError(error); });
    };
    ApiService.prototype.head = function (url, options) {
        return this.http.head(url, options);
    };
    ApiService.prototype.options = function (url, options) {
        return this.http.options(url, options);
    };
    ApiService.urlSearchQueryParam = 'search';
    ApiService.urlSearchFieldsParam = 'searchFields';
    ApiService.urlOrderParam = 'orderBy';
    ApiService.urlSortParam = 'sortedBy';
    ApiService.urlFilterParam = 'filter';
    ApiService.urlIncludeParam = 'include';
    return ApiService;
}());
export { ApiService };
//# sourceMappingURL=api.service.js.map