import { ITransformerMap } from "./itransformer.map";
import { Aliases } from "./aliases";
import { ITransformer } from "./itransformer";
export declare abstract class Transformer<Model> implements ITransformer<Model> {
    private transformersMap;
    private withRelations;
    private response;
    private aliasesMap;
    /**
     *
     * @param transformersMap
     * @param withRelations
     */
    constructor(transformersMap: ITransformerMap, withRelations?: boolean);
    /**
     * Create aliases map.
     *
     * @param aliases
     * @returns {AliasMap}
     */
    private static createAliasesMap(aliases);
    /**
     * Aliases map.
     * Using for setup transformer aliases for different entity types.
     */
    abstract aliases(): Aliases | undefined;
    /**
     * Transform method.
     *
     * @param data
     */
    abstract transform(data: any): Model;
    /**
     * Transform response data to model entity.
     *
     * @param response
     * @returns {Model}
     */
    call(response: any): Model | Model[] | undefined;
    /**
     * Parse main model data.
     *
     * @param data
     * @returns {Model}
     */
    private parse(data);
    /**
     * Parse relations list.
     *
     * @param relations
     * @param model
     */
    private parseRelations(relations, model);
    /**
     * Parse the relation.
     *
     * @param data
     * @param model
     * @param isArray
     */
    private parseRelation(data, model, isArray?);
    /**
     * Find an alias for model type.
     *
     * @param type
     * @returns {undefined|string}
     */
    private findAlias(type);
    /**
     * Find model include data.
     *
     * @param type
     * @param id
     * @returns {any}
     */
    private findInclude(type, id);
    /**
     * Parse model include data.
     *
     * @param include
     * @param model
     * @param alias
     * @param isArray
     */
    private parseInclude(include, model, alias?, isArray?);
}
