import {IApiResponse} from "./iapi-response";
import {ApiService} from "./api.service";
import {ITransformer} from "./itransformer";
import {IPagination} from "./ipagination";
import {ILinks} from "./ilinks";

export class ApiResponse<Model> implements IApiResponse<Model> {
    constructor(private service: ApiService<Model>,
                private response: any) {
    }

    private transform(data: any): Model | Model[] | undefined {
        let transformer: ITransformer<Model> = this.service.transformer();
        return transformer.call(data);
    }

    public fetch(): Model | Model[] | undefined {
        return this.transform(this.getResponse());
    }

    public getPagination(): IPagination {
        return <IPagination>this.getMeta().pagination;
    }

    public getLinks(): ILinks {
        return <ILinks>this.getResponse().links;
    }

    public getIncluded(): any {
        return this.getResponse().included;
    }

    public getMeta(): any {
        return this.getResponse().meta;
    }

    public getData(): any {
        return this.getResponse().data;
    }

    public getResponse(): any {
        return this.response;
    }
}