import {Factory} from "./factory";
import {ApiCollection} from "./api.collection";
import {Order} from "./order";
import {OrderCollection} from "./order.collection";
import {SortCollection} from "./sort.collection";

export class OrderFactory extends Factory<ApiCollection<Order>> {
    private order: any;

    constructor(order: Order);
    constructor(order: Order[]);
    constructor(order: any) {
        super();

        if (!(order instanceof Order || order instanceof Array))
            throw new TypeError('Order argument must be a type "Order" or "Order[]".');

        if (!(order instanceof Array))
            this.order = [order];
        else
            this.order = order;
    }

    public create(type: string): ApiCollection<Order> {
        switch (type) {
            case OrderFactory.TYPE_QUERY:
                return new OrderCollection(this.order);
            case OrderFactory.TYPE_FIELDS:
                return new SortCollection(this.order);
            default:
                throw new Error('Order factory support only: "query" or "fields" type.');
        }
    }
}