import {Search} from "./search";
import {Factory} from "./factory";
import {ApiCollection} from "./api.collection";
import {SearchCollection} from "./search.collection";
import {SearchFieldCollection} from "./search-field.collection";

export class SearchFactory extends Factory<ApiCollection<Search>> {
    private search: Search[];

    constructor(search: Search);
    constructor(search: Search[]);
    constructor(search: any) {
        super();

        if (!(search instanceof Search || search instanceof Array))
            throw new TypeError('Search argument must be a type "Search" or "Search[]".');

        if (!(search instanceof Array))
            this.search = [search];
        else
            this.search = search;
    }

    public create(type: string): ApiCollection<Search> {
        switch (type) {
            case SearchFactory.TYPE_QUERY:
                return new SearchCollection(this.search);
            case SearchFactory.TYPE_FIELDS:
                return new SearchFieldCollection(this.search);
            default:
                throw new Error('Search factory support only: "query" or "fields" type.');
        }
    }
}