import {Injectable, InjectionToken, Injector} from "@angular/core";
import {ITransformer} from "./itransformer";
import {IEntity} from "./ientity";
import {ITransformerDataMap, ITransformerDataMapEntry} from "./itransformer-data.map";
import {ITransformerMap} from "./itransformer.map";

export const TRANSFORMERS_DATA_MAP: ITransformerDataMap = <ITransformerDataMap>{
    // 'product': {
    //     token: new InjectionToken<ProductTransformer>('app.transformers.product'),
    //     transformerClass: ProductTransformer
    // },

    // OR

    // 'product': new InjectionToken<ProductTransformer>('app.transformers.product'),
};

@Injectable()
export class TransformerMap implements ITransformerMap {
    private static map: ITransformerDataMap = TRANSFORMERS_DATA_MAP;

    constructor(private injector: Injector) {}

    /**
     * Put the transformer entry to map.
     *
     * @param key
     * @param entry
     */
    public put(key: string, entry: ITransformerDataMapEntry): void;
    /**
     * Put token and transformer class to map.
     * @param key
     * @param token
     * @param transformerClass
     */
    public put(key: string, token: InjectionToken<ITransformer<IEntity>>, transformerClass?: new (transformersMap: ITransformerMap) => ITransformer<IEntity>): void;
    /**
     * Put the transformer entry to map.
     *
     * @param args
     */
    public put(...args: any[]): void {
        let key: string = args[0];
        let entryToken: ITransformerDataMapEntry | InjectionToken<ITransformer<IEntity>> = args[1];
        let transformerClass: new (transformersMap: ITransformerMap) => ITransformer<IEntity> = args[2];

        let entry: ITransformerDataMapEntry;

        if (entryToken instanceof InjectionToken)
            entry = TransformerMap.createEntry(entryToken, transformerClass);
        else
            entry = entryToken;

        TransformerMap.map[key] = entry;
    }

    /**
     * Create transformer map entry.
     *
     * @param token Injection token.
     * @param transformerClass Transformer class.
     * @returns {ITransformerDataMapEntry}
     */
    private static createEntry(token: InjectionToken<ITransformer<IEntity>>,
                               transformerClass?: new (transformersMap: ITransformerMap) => ITransformer<IEntity>): ITransformerDataMapEntry {

        return <ITransformerDataMapEntry>{
            token: token,
            transformerClass: transformerClass
        };
    }

    /**
     * Inject and get the transformer instance by key.
     *
     * @param key
     * @returns {ITransformer<IEntity>}
     */
    public get(key: string): ITransformer<IEntity> {
        try {
            return this.injector.get(TransformerMap.map[key].token);
        } catch (e) {
            throw new Error(`Can't inject transformer by key "${key}". Maybe transformer not registered in the map.`);
        }
    }
}

export const TRANSFORMERS_MAP = new InjectionToken<ITransformerMap>('app.transformers-map');
