import {TRANSFORMERS_MAP, TransformerMap, TRANSFORMERS_DATA_MAP} from "./transformer-map.service";
import {Injector, ModuleWithProviders, NgModule, Provider} from "@angular/core";
import {ITransformerDataMap} from "./itransformer-data.map";
import {ITransformerMap} from "./itransformer.map";
import {ITransformer} from "./itransformer";
import {IEntity} from "./ientity";

export function transformerMapFactory(injector: Injector): ITransformerMap {
    return new TransformerMap(injector);
}

export function transformerFactory(transformersMap: ITransformerMap, map: ITransformerDataMap, key: string): ITransformer<IEntity> {
    return new map[key].transformerClass(transformersMap);
}

@NgModule({
    providers: TransformerModule.providers
})
export class TransformerModule {
    private static providers: Provider[] = [{
        provide: TRANSFORMERS_MAP,
        useFactory: transformerMapFactory,
        deps: [Injector]
    }];

    static extend(map: ITransformerDataMap, withProviders: boolean = true): ModuleWithProviders {
        Object.assign(TRANSFORMERS_DATA_MAP, map);

        if (withProviders)
            this.extendProviders(map);

        return {
            ngModule: TransformerModule,
            providers: this.providers
        };
    }

    private static extendProviders(map: ITransformerDataMap): void {
        let keys = Object.getOwnPropertyNames(map);
        for (let key of keys) {
            const provider: Provider = {
                provide: map[key].token,
                useFactory: transformerFactory,
                deps: [TRANSFORMERS_MAP, map, key]
            };

            this.providers.push(provider);
        }
    }
}